#!/usr/bin/perl -w
#
# STRindexer.pl v1.1
# Last modified May 24, 2005 16:05
# (c) Olaf R.P. Bininda-Emonds
#
# Input: html output file produced from the Perl script PerlEQ.pl
#
# Output: A file (*_index.txt) listing the index taxa for all equivalents where the two must
#         originate from the same node (i.e., all c* category matches), thus allowing the
#		  equivalent to be potentially re-added to the tree (e.g., using reverseSTR.pl)
#
# Usage: STRindexer.pl -d<filename> [-e<m|u|b>] [-h]
#	options: -d<filename> = name of html file containing results of STR analysis from PerlEQ
#			 -e<m|u|b> = data to be considered in determining equivalents: m = missing data only (default);
#						 u = missing and uninformative data; b = both
#			 -h = print this message and quit

use strict;

# Set default parameters
	# I/O parameters
		my ($STRfile, $dataSource);
		
	# Equivalence parameters
		my $equivCategory;	# Options are missing (4), uninformative (7), or both
		my %equivRead = ('4' => '1', '7' => '0');
		my %equivName = ('4' => 'missing', '7' => 'missUninf');

	# Miscellaneous parameters
		my $verbose = 0;
		my $debug = 0;

# Get command line input
	for (my $i = 0; $i <= $#ARGV; $i++)
		{
		if ($ARGV[$i] =~ /-d(.*)/)
			{
			$STRfile = $1;
			($dataSource = $STRfile) =~ s/\.htm$//;
			}
		elsif ($ARGV[$i] =~ /-e([mub])/i)
			{
			my $dataType = $1;

			if ($dataType eq "m")
				{
				$equivRead{4} = 1;
				$equivRead{7} = 0;
				}
			if ($dataType eq "u")
				{
				$equivRead{4} = 0;
				$equivRead{7} = 1;
				}
			if ($dataType eq "b")
				{
				$equivRead{4} = 1;
				$equivRead{7} = 1;
				}
			}
		elsif ($ARGV[$i] eq "-h")
			{
			print "Usage: STRindexer.pl -d<filename> [-e<m|u|b>] [-h]\n";
			print "Options: -d<filename> = name of html file containing results of STR analysis from PerlEQ\n";
			print "         -e<m|u|b> = data to be considered in determining equivalents: m = missing data only (default);\n";
			print "                     u = missing and uninformative data; b = both\n";
			print "         -h = print this message and quit\n";
			exit(0);
			}
		else
			{
			print "Don't understand argument: $ARGV[$i]\n";
			print "Usage: STRindexer.pl -d<filename> [-e<m|u|b>] [-h]\n";
			exit(1); 
			}
		}

	if (not defined $STRfile or ($equivRead{4} == 0 and $equivRead{7} == 0))
		{
		print "ERROR: Must provide name of html file containing results of STR analysis from PerlEQ\n" if (not defined $STRfile);
		print "ERROR: Must specify data types used to determine equivalents \n" if ($equivRead{4} == 0 and $equivRead{7} == 0);
		print "Usage: STRindexer.pl -d<filename> [-e<m|u|b>] [-h]\n";
		exit(1); 
		}

# Read in and process STR results
	print "Reading in results of STR analysis from file $STRfile ...\n\n";
	setLineBreak($STRfile);
	open (STR, "<$STRfile") or die "Cannot open file containing results of STR analysis, $STRfile\n";
		my $readFlag = 0;
		my $columnCount = 0;
		my $equivType;
		my $indexTaxon;
		my %index;
		my (@equivList, %equivPresent);
		
		while (<STR>)
			{
			chomp;
			my $lineRead = $_;
			next unless ($lineRead);

			if ($lineRead =~ /\<h2\>(\d)\. Equivalents/i)	# Set conditions to read in desired blocks
				{
				$equivType = $1;
				$readFlag = 1 if ($equivRead{$equivType});
				next;
				}

			next unless ($readFlag);
			next if ($lineRead =~ /Shows no equivalence/i);
			next unless ($lineRead =~ /^\<tr/i or $lineRead =~ /^\s+\<td class/i or $lineRead =~ /^\<\/table>/i);

			print "$lineRead\n" if ($debug);

			if ($lineRead =~ /^\<tr/i)
				{
				$columnCount = 0;
				next;
				}

			if ($lineRead =~ /^\s+\<td class/i)
				{
				$columnCount++;
					next if ($columnCount == 2);	# Skip information about missing/uncertain data
					
				if ($columnCount == 1)
					{
					next if ($lineRead =~ /&nbsp;/);	# Continuation of species data in a new row; skip
					$indexTaxon = $lineRead;
					 	$indexTaxon =~ s/^\s+\<td class=\"nexusEq\"\>\<i\>//;	# Clear html formatting
					 	$indexTaxon =~ s/\<\/i\>\<\/td\>$//;
					 	print "\tIndex taxon: $indexTaxon\n" if ($debug);
					}

				if ($columnCount == 3)
					{
					next unless ($lineRead =~ /\(C\*?\)/i);	# Doesn't contain any species of right category; skip
					my $equivText = $lineRead;
					 	$equivText =~ s/^\s+\<td class=\"nexusEq\"\>//;	# Clear html formatting
					 	$equivText =~ s/\<\/td\>$//;
					 	$equivText =~ s/\<i\>//g;
					 	$equivText =~ s/\<\/i\>//g;
					 	print "\tEquivalents to $indexTaxon: $equivText\n" if ($debug);
					 	
					 foreach my $equiv (split(/,\s+/, $equivText))
					 	{
					 	print "\t\t$equiv\n" if ($debug);
						next unless ($equiv =~ /C\*/);
						$equiv =~ s/\(\w\*\)//;

						push @ {$index{$equiv} }, $indexTaxon;
						if (not defined $equivPresent{$equiv})
							{
							push @equivList, $equiv;
							$equivPresent{$equiv} = 1;
							}
					 	}
					}
				}
				
			if ($lineRead =~ /^\<\/table>/i)	# End of block; close readFlag and print results
				{
				$readFlag = 0;

				@equivList = sort @equivList;
				my $outFile .= $dataSource . "_$equivName{$equivType}" . "_index.txt";
					print "Outputting results to file $outFile ...\n";
					open (OUT, ">$outFile") or die "Cannot open file, $outFile, to print results\n";
						print OUT "Taxon\tIndex taxon (C*)\n\n";
						if (scalar(@equivList))
							{
							foreach my $equiv (@equivList)
								{
								print OUT "$equiv\t";
								print OUT "\t$_" foreach (@ {$index{$equiv} });
								print OUT "\n";
								}
							}
						else
							{
							print OUT "No suitable equivalents found";
							}
							
						print OUT "\n\n===================================================================\n";
						print OUT "(Abbreviated) KEY to Taxonomic Equivalence\n";
						print OUT "Note: equivalence scores have been transferred to the index taxon\n\n";
						print OUT "Equivalence Scores:\n";
						print OUT "C - Potential equivalents (asymmetric all one way)\n";
						print OUT "    Equivalent can be safely deleted if Index is retained\n\n";
						print OUT "Suffixes:\n";
						print OUT "* - Equivalent taxon must originate from same node as Index taxon in\n";
						print OUT "    any MPT (assuming no arbitrary resolutions)\n";
					close OUT;
				
				# Clear variables for next block
					undef @equivList;
					undef %index;
					undef %equivPresent;
				}
			}
	close STR;
	
exit(0);

### Subroutines used in the program

sub setLineBreak	# Check line breaks of input files and set input record separator accordingly
	{
	my $inFile = shift;
	$/ ="\n";
	open (IN, "<$inFile") or die "Cannot open $inFile to check form of line breaks.\n";
		while (<IN>)
			{
			if ($_ =~ /\r\n/)
				{
				print "\tDOS line breaks detected ...\n" if ($verbose);
				$/ ="\r\n";
				last;
				}
			elsif ($_ =~ /\r/)
				{
				print "\tMac line breaks detected ...\n" if ($verbose);
				$/ ="\r";
				last;
				}
			else
				{
				print "\tUnix line breaks detected ...\n" if ($verbose);
				$/ ="\n";
				last;
				}
			}
	close IN;
	}

# Version history
#
#	v1.1 (May 24, 2005)
#		- updated to read new html output from PerlEQ.pl v1.0b12; no longer reads text output (.txq)
#		- added command line interface
#		- added option to specify how equivalents are determined (missing data only,
#		  missing data + uninformative characters, or both)
#
#	v1.0 (January 26, 2005)
#		- initial release
